<?php

///////////////////////////////////////////////////////////////////////////

require_once 'lib/default_dune_plugin.php';
require_once 'lib/utils.php';
require_once 'lib/default_archive.php';


require_once 'lib/tv/tv_group_list_screen.php';
require_once 'lib/tv/tv_channel_list_screen.php';
require_once 'lib/tv/tv_favorites_screen.php';

require_once 'lib/vod/vod_list_screen.php';
require_once 'lib/vod/vod_movie_screen.php';
require_once 'sunduk_vod_series_list_screen.php';
require_once 'sunduk_vod_format_list_screen.php';
require_once 'lib/vod/vod_favorites_screen.php';

require_once 'sunduk.php';
require_once 'sunduk_session.php';
require_once 'sunduk_setup_screen.php';
require_once 'sunduk_exception.php';

require_once 'sunduk_entry_handler.php';

require_once 'sunduk_main_category_list_screen.php';

require_once 'sunduk_tv.php';
require_once 'sunduk_tv_channel_list_screen.php';

require_once 'sunduk_vod.php';
require_once 'sunduk_vod_root_screen.php';
require_once 'sunduk_vod_list_screen.php';
require_once 'sunduk_vod_genres_screen.php';
require_once 'sunduk_vod_search_screen.php';

///////////////////////////////////////////////////////////////////////////

class SundukPlugin extends DefaultDunePlugin {

    private $session;
    private $entry_handler;

    public function __construct() {
        $this->session = new SundukSession($this);
        $this->entry_handler = new SundukEntryHandler($this->session);
        $this->tv = new SundukTv($this->session);
        $this->vod = new SundukVod($this->session);

        $tv_folder_views = $this->get_tv_folder_views();
        $tv_group_views = $this->get_tv_group_views();

        $this->add_screen(new SundukMainCategoryListScreen($this->session, $this->get_main_category_folder_views()));

        $this->add_screen(new SundukSetupScreen($this->session));

        $this->add_screen(
                new TvGroupListScreen($this->tv, $tv_group_views));

        $this->add_screen(
                new TvChannelListScreen($this->tv, $tv_folder_views));

        $this->add_screen(
                new TvFavoritesScreen($this->tv, $tv_folder_views));

        $this->add_screen(
                new SundukVodRootScreen($this->session, $tv_folder_views));

        $this->add_screen(
                new SundukVodListScreen($this->session, $this->vod));

        $this->add_screen(new VodMovieScreen($this->vod));

        $this->add_screen(new SundukVodSeriesListScreen($this->vod));
        
        $this->add_screen(new SundukVodFormatListScreen($this->vod));

        $this->add_screen(new VodFavoritesScreen($this->vod));

        $this->add_screen(new SundukVodSearchScreen($this->vod));

        $this->add_screen(new SundukVodGenresScreen($this->vod));
    }

    public function unset_login() {
        $this->session->unset_login();
        $this->tv->unload_channels();
        $this->vod->clear_movie_cache();
        $this->vod->clear_genre_cache();

        DefaultArchive::clear_cache();
    }

    private function get_tv_folder_views() {
        return array(
            array
                (
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 5,
                    ViewParams::num_rows => 4,
                    ViewParams::paint_details => false,
                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 210,
                    ViewParams::sandwich_height => 150,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => true,
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_CENTER,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_paint_caption => false,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array
                    (
                    ViewItemParams::icon_path => $this->session->get_icon('template.png'),
                    ViewItemParams::item_paint_caption_within_icon => true,
                    ViewItemParams::item_caption_within_icon_color => 3,
                ),
            ),
            array
                (
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 4,
                    ViewParams::num_rows => 4,
                    ViewParams::paint_details => true,
                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 210,
                    ViewParams::sandwich_height => 150,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => true,
                    ViewParams::paint_details => true,
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_CENTER,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_paint_caption => false,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array
                    (
                    ViewItemParams::icon_path => $this->session->get_icon('template.png'),
                    ViewItemParams::item_paint_caption_within_icon => true,
                    ViewItemParams::item_caption_within_icon_color => 'yellow',
                ),
            ),
            array
                (
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 5,
                    ViewParams::num_rows => 3,
                    ViewParams::paint_details => false,
                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 210,
                    ViewParams::sandwich_height => 150,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => true,
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_CENTER,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_paint_caption => true,
                    ViewItemParams::item_caption_sel_dy => 15,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array
                    (
                    ViewItemParams::icon_path => $this->session->get_icon('template.png'),
                    ViewItemParams::item_paint_caption_within_icon => false,
                    ViewItemParams::item_caption_within_icon_color => 'yellow',
                ),
            ),
            array
                (
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 5,
                    ViewParams::num_rows => 3,
                    ViewParams::paint_details => false,
                    ViewParams::paint_sandwich => true,
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 210,
                    ViewParams::sandwich_height => 150,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => true,
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_CENTER,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_paint_caption => false,
                    ViewItemParams::icon_scale_factor => 1.2,
                    ViewItemParams::icon_sel_scale_factor => 1.5,
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array
                    (
                    ViewItemParams::icon_path => $this->session->get_icon('template.png'),
                    ViewItemParams::item_paint_caption_within_icon => true,
                    ViewItemParams::item_caption_within_icon_color => 'yellow',
                ),
            ),
            array
                (
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 2,
                    ViewParams::num_rows => 10,
                    ViewParams::paint_details => true,
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 10,
                    ViewItemParams::icon_dy => -5,
                    ViewItemParams::icon_width => 63,
                    ViewItemParams::icon_height => 45,
                    ViewItemParams::item_caption_width => 485,
                    ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                    ViewItemParams::icon_path => $this->session->get_icon('template.png'),
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array(),
            )
        );
    }

    private function get_tv_group_views() {
        return array(
            array
                (
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 2,
                    ViewParams::num_rows => 10,
                    ViewParams::paint_details => false,
                    ViewParams::zoom_detailed_icon => false,
                    ViewParams::extra_content_objects =>
                    'label{x=50}{y=760}{text=Русскоязычное телевидение Sunduk.TV}{color=10}{font_size=1}{iconpath=plugin_file://icons/logo.aai}',
                //        'Image{x=180}{y=550}{url=plugin_file://icons/logo.aai}',
                //    'picture{x=50}{y=600}{=fjkgdkfgdg}',
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 10,
                    ViewItemParams::icon_dy => -5,
                    ViewItemParams::icon_width => 84,
                    ViewItemParams::icon_height => 48,
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::item_caption_width => 700,
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array
                    (
                    ViewItemParams::icon_path => $this->session->get_icon('template.png'),
                ),
            )
        );
    }

    private function get_main_category_folder_views() {
        return array(
            array
                (
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 3,
                    ViewParams::num_rows => 2,
                    ViewParams::paint_details => false,
                    ViewParams::paint_sandwich => false,
                    ViewParams::item_width => 350,
                    ViewParams::item_height => 350,
                    ViewParams::background_path=> $this->session->get_icon('bg.jpg'),
                    ViewParams::paint_path_box => true,
                    ViewParams::background_x => 0,
                    ViewParams::background_y => 0,
                    ViewParams::background_height=> 1080,
                    ViewParams::background_width=> 1920,
                    ViewParams::background_order => 0,
//                    ViewParams::optimize_full_screen_background => true,
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_CENTER,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_paint_caption => true,
                    ViewItemParams::item_caption_color => 0,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.0,
//                    ViewItemParams::icon_width => 300,
//                    ViewItemParams::icon_height => 300,
//                    ViewItemParams::icon_sel_width => 300,
//                    ViewItemParams::icon_sel_height => 300,
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array
                    (
                    ViewItemParams::icon_path => $this->session->get_icon('template.png'),
                    ViewItemParams::item_paint_caption_within_icon => true,
                    ViewItemParams::item_caption_within_icon_color => 3,
                ),
            )
        );
    }

}
