<?php

class SundukVodFormatListScreen extends AbstractPreloadedRegularScreen implements UserInputHandler {

    const ID = 'vod_format';

    private $vod;

    public function __construct(Vod $vod) {

        $this->vod = $vod;
        parent::__construct(self::ID, $this->get_folder_views());
        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    public function get_action_map(MediaURL $media_url, &$plugin_cookies) {

        return array
            (
            GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action($this, 'enter'),
            GUI_EVENT_KEY_PLAY => UserInputHandlerRegistry::create_action($this, 'play'),
        );
    }

    public static function get_media_url_str($movie_id, $series_id) {

        return MediaURL::encode(
                        array(
                            'screen_id' => self::ID,
                            'movie_id' => $movie_id,
                            'series_id' => $series_id
        ));
    }

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies) {

        $this->vod->folder_entered($media_url, $plugin_cookies);

        $movie = $this->vod->get_loaded_movie($media_url->movie_id, $plugin_cookies);
        if ($movie === null) {
            return array();
        }

        $series = null;
        foreach($movie->series_list as $s){
            $series = $s;
            if($s->id == $media_url->series_id){
                break;
            }
        }
        if ($series === null) {
            return array();
        }

        $items = array();
        foreach ($series->formates as $format) {
            $items[] = array
                (
                PluginRegularFolderItem::media_url =>
                MediaURL::encode(
                        array
                            (
                            'screen_id' => 'vod_format',
                            'movie_id' => $movie->id,
                            'series_id' => $series->id,
                            'format_id' => $format
                )),
                PluginRegularFolderItem::caption => $format,
                PluginRegularFolderItem::view_item_params => array
                    (
                    ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                ),
            );
        }

        return $items;
    }

    public function get_handler_id() {

        return self::ID;
    }

    public function handle_user_input(&$user_input, &$plugin_cookies) {

        ob_start();
        var_dump($user_input);
        hd_print(ob_get_clean());
        $media_url = MediaURL::decode($user_input->selected_media_url);
        $movie = $this->vod->get_loaded_movie($media_url->movie_id, $plugin_cookies);
        ob_start();
        var_dump($media_url);
        hd_print(ob_get_clean());
        $series = null;
        foreach($movie->series_list as $s){
            $series = $s;
            if($s->id == $media_url->series_id){
                break;
            }
        }
        if (empty($media_url->format_id)) {
            $series->playback_url = $series->playback_url_original . '&format=' . $series->formates[0];
        }
        else {
            $series->playback_url = $series->playback_url_original . '&format=' . $media_url->format_id;
        }
        ob_start();
        var_dump($series);
        hd_print(ob_get_clean());
        return ActionFactory::vod_play();
    }

    private function get_folder_views() {

        return array(
            array
                (
                PluginRegularFolderView::view_params => array
                    (
                    ViewParams::num_cols => 1,
                    ViewParams::num_rows => 12,
                ),
                PluginRegularFolderView::base_view_item_params => array
                    (
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 10,
                    ViewItemParams::icon_dy => -5,
                    ViewItemParams::item_caption_dx => 60,
                    ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai'
                ),
                PluginRegularFolderView::not_loaded_view_item_params => array(),
            ),
        );
    }

    public function get_archive(MediaURL $media_url) {

        return $this->vod->get_archive($media_url);
    }

}
